/*
** Module   : BrowseModel.java
** Abstract : data model for Browse widget
**
** Copyright (c) 2005-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------------------------------
** 001 SVG 20050803   @21995 Initial version.
** 002 SIY 20050831   @22527 Formatting and spelling cleanups. 
** 003 SIY 20050926   @22825 Added isQueryOffEnd() and isNewRow() methods.
** 004 SIY 20051201   @23591 Removed isQueryOffEnd() method.
** 005 SIY 20060315   @25124 Changes for complete BROWSE implementation.
** 006 GES 20060808   @28494 Use int instead of long in the APIs.
** 007 SIY 20061105   @30986 Added getCurrentRow() method.
** 008 SIY 20070504   @33419 Updated getRowCount().
** 009 SIY 20111101          Moved from CHUI package. Minor 
**                           cleanups/refactorings in preparation to GUI client.
** 010 SVL 20150212          Modified getRows signature.
** 011 SVL 20150411          Added isRowAvailable.
** 012 SVL 20150503          Added resetColumnsCache.
** 013 SVL 20150608          Added getEditingFillIns.
*/
package com.goldencode.p2j.ui.client;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.util.*;

/**
 * Data model for BROWSE widget.
 */
public interface BrowseModel
{
   /**
    * Return information about columns.
    * 
    * @return  A meta information about columns.
    */
   public BrowseColumnConfig[] getColumns();

   /**
    * Get configurations of fill-ins used for in-browse editing.
    *
    * @return   Array of configurations of fill-in widgets which are used for in-browse editing.
    *           An array element is configuration of the fill-in widget for the corresponding
    *           editable column or <code>null</code> if the column is not editable. Return value
    *           is <code>null</code> if the browse is read-only.
    */
   public FillInConfig[] getEditingFillIns();

   /**
    * Get number of available rows starting from the given one.
    *
    * @param   startRow
    *          Initial row to start count rows from.
    *
    * @return  Number of available rows.
    */
   public int getRowCount(int startRow);

   /**
    * Retrieves several subsequent rows from the database.
    *
    * @param    startRowIndex
    *           Index of the first row to retrieve.
    * @param    count
    *           Maximum count of rows to return.
    * @param    forward
    *           <code>true</code> if rows are retrieved in the forward direction,
    *           <code>false</code> if rows are retrieved in the backward direction,
    *
    * @return   Array of rows. Each row is array of cell values, but the first element of the row
    *           array is 0-based index of the row in the browse model.
    *           Note that if rows are fetched in the backward direction, the resulting rows are
    *           returned in the straight order starting from the row with the least row number.
    *           If no query specified, an array of size 0 is returned.
    */
   public BaseDataType[][] getRows(int startRowIndex, int count, boolean forward);
   
   /**
    * Updates selected cells in the given row.
    * 
    * @param   rowIndex
    *          Index of the row to update.
    * @param   cellIndices
    *          Cells for update indexes.
    * @param   cellValues
    *          Cells for update new values.
    * 
    * @throws  DataValidationException
    *          New values don't passed database validation rules.
    */
   public void updateRow(int            rowIndex,
                         int[]          cellIndices,
                         BaseDataType[] cellValues)
   throws DataValidationException;

   /**
    * Returns <code>true</code> if specified row is new and is not present
    * in the data base.
    * 
    * @param   rowIndex
    *          Index of the row to return information about.
    * @return  <code>true</code> if row is new and <code>false</code>
    *          otherwise.
    */
   public boolean isNewRow(int rowIndex);

   /**
    * Inform server about current row in the browse.
    * 
    * @param   rowIndex
    *          0-based index of the current row.
    *          
    * @return  New current row.
    */
   public int setCurrentRow(int rowIndex);

   /**
    * Get current row in the model.
    * 
    * @return  The current row index in the model.
    */
   public int getCurrentRow();

   /**
    * Determines if the specified browse row is "available", i.e. it is not deleted.
    *
    * @param  rowIndex
    *         0-base index of the target row in the browse model.
    *
    * @return <code>true</code> if the row is "available", i.e. it is not deleted.
    */
   public boolean isRowAvailable(int rowIndex);

   /**
    * Reset cached information about columns.
    */
   public void resetColumnsCache();
}
